@@ -1,16 +1,18 @@
-from typing import Optional, Literal, TypedDict
+from typing import Optional, Literal, TypedDict, Union
 from .StandardConstants import *
 from lxml import etree
 import datetime
-
+import os
 
 
 class InternetRelationDict(TypedDict):
+    "the type for the content of the Internet attribute"
     value: INTERNET_RELATION_VALUE_LITERAL
     bandwidth_mbitps: Optional[int]
 
 
 class TranslateableTag:
+    "Represents a translatable tag"
     def __init__(self) -> None:
         self._text = ""
         self._translations: dict[str, str] = {}
@@ -53,7 +55,7 @@ class TranslateableTag:
             translation_tag.set("{http://www.w3.org/XML/1998/namespace}lang", key)
             translation_tag.text = value
 
-    def clear(self):
+    def clear(self) -> None:
         """Resets all data"""
         self._text = ""
         self._translations.clear()
@@ -63,16 +65,17 @@ class TranslateableTag:
 
 
 class TranslateableList:
+    "Represents a translatable list"
     def __init__(self) -> None:
         self.default_list = []
         self.translated_lists = {}
 
     def get_default_list(self) -> list[str]:
-        "Returns a list with the defualt items"
+        "Returns a list with the default items"
         return self.default_list
 
-    def load_tag(self, tag: etree.Element):
-        """Loads a Tag"""
+    def load_tag(self, tag: etree.Element) -> None:
+        "Loads a Tag Only for internal use."
         if tag.get("{http://www.w3.org/XML/1998/namespace}lang") is None:
             for i in tag.getchildren():
                 if i.get("{http://www.w3.org/XML/1998/namespace}lang") is None:
@@ -88,46 +91,56 @@ class TranslateableList:
                 self.translated_lists[tag.get("{http://www.w3.org/XML/1998/namespace}lang")].append(i.text.strip())
 
     def write_all_tag(self, parent_tag: etree.Element, tag_name: str) -> None:
+        "Writes the XML tags. Onyl for internal use."
         for i in range(len(self.default_list)):
             default_tag = etree.SubElement(parent_tag, tag_name)
             default_tag.text = self.default_list[i]
 
-    def clear(self):
+    def clear(self) -> None:
         """Resets all data"""
         self.default_list.clear()
         self.translated_lists.clear()
 
 
 class DescriptionItem:
-    def get_type(self) -> str:
+    "The Interface for a Description Item"
+    def get_type(self) -> Literal["paragraph", "unordered-list", "ordered-list"]:
+        "Retutns the Type of the Item"
         return "none"
 
     def load_tags(self, tag_list: list[etree.Element]) -> None:
+        "Loads teh XML tags into the Elemnt. Only for internal use."
         raise NotImplementedError
 
     def get_tags(self, parent_tag: etree.Element) -> None:
+        "Get the XML Tag from the Element. Only for internal use."
         raise NotImplementedError()
 
     def get_translated_tag(self, lang: Optional[str]) -> etree.Element:
+        "Loads the tag for a given language"
         raise NotImplementedError()
 
 
 class DescriptionParagraph(DescriptionItem):
+    "Represents a paragraph <p> in the Description"
     def __init__(self) -> None:
         self.content = TranslateableTag()
         """The Text of the Paragraph"""
 
-    def get_type(self) -> str:
+    def get_type(self) -> Literal["paragraph", "unordered-list", "ordered-list"]:
+        "Retutns the Type of the Item"
         return "paragraph"
 
     def load_tags(self, tag_list: list[etree.Element]) -> None:
+        "Loads teh XML tags into the Elemnt. Only for internal use."
         self.content.load_tags(tag_list)
 
     def get_tags(self, parent_tag: etree.Element) -> None:
+        "Get the XML Tag from the Element. Only for internal use."
         self.content.write_tags(parent_tag, "p")
 
     def get_translated_tag(self, lang: Optional[str]) -> etree.Element:
-        "Get the translated tag"
+        "Loads the tag for a given language"
         paragraph_tag = etree.Element("p")
         if lang is None:
             paragraph_tag.text = self.content.get_default_text()
@@ -137,38 +150,44 @@ class DescriptionParagraph(DescriptionItem):
 
 
 class DescriptionList(DescriptionItem):
+    "Represents a list <ul>/<ol> in the Description"
     def __init__(self, list_type: str) -> None:
         self._list_type = list_type
 
         self.content: TranslateableList = TranslateableList()
         "The list"
 
-    def get_type(self) -> str:
+    def get_type(self) -> Literal["paragraph", "unordered-list", "ordered-list"]:
+        "Retutns the Type of the Item"
         if self.list_type == "ul":
             return "unordered-list"
         else:
             return "ordered-list"
 
     def load_tags(self, tag_list: list[etree.Element]) -> None:
+        "Loads teh XML tags into the Elemnt. Only for internal use."
         self.content.load_tag(tag_list)
 
     def get_tags(self, parent_tag: etree.Element) -> None:
+        "Get the XML Tag from the Element. Only for internal use."
         list_tag = etree.SubElement(parent_tag, self._list_type)
         self.content.write_all_tag(list_tag, "li")
 
     def get_translated_tag(self, lang: Optional[str]) -> etree.Element:
-        "Get the translated tag"
+        "Loads the tag for a given language"
         list_tag = etree.Element(self._list_type)
         self.content.write_all_tag(list_tag, "li")
         return list_tag
 
 
 class Description:
+    "Represents a <description> tag"
     def __init__(self) -> None:
         self.items: list[DescriptionItem] = []
         """All Description Items"""
 
-    def load_tags(self, tag: etree.Element):
+    def load_tags(self, tag: etree.Element) -> None:
+        "Load a XML tag. Onyl for internal use."
         paragraph_list: list[etree.Element] = []
         for i in tag.getchildren():
             if i.tag == "p":
@@ -198,6 +217,7 @@ class Description:
             self.items.append(paragraph_item)
 
     def get_tags(self, parent_tag: etree.Element) -> None:
+        "Writes a Description tag. Only for internal use."
         description_tag = etree.SubElement(parent_tag, "description")
         for i in self.items:
             i.get_tags(description_tag)
@@ -219,6 +239,7 @@ class Description:
 
 
 class Release:
+    "Represents a <release> tag"
     def __init__(self) -> None:
         self.version: str = ""
         "The version"
@@ -229,7 +250,7 @@ class Release:
         self.description = Description()
         "The description"
 
-    def load_tag(self, tag: etree.Element):
+    def load_tag(self, tag: etree.Element) -> None:
         "Loads a release tag"
 
         self.version = tag.get("version") or ""
@@ -250,6 +271,7 @@ class Release:
 
 
 class Image:
+    "Represents a <image> tag"
     def __init__(self) -> None:
         self.url: str = ""
         "The image URL"
@@ -266,7 +288,7 @@ class Image:
         self.language: Optional[str] = None
         "The language"
 
-    def load_tag(self, tag: etree.Element):
+    def load_tag(self, tag: etree.Element) -> None:
         "Loads a image tag"
         self.url = tag.text.strip()
         self.type = tag.get("type", "source")
@@ -285,6 +307,7 @@ class Image:
 
 
 class Screenshot:
+    "Represents a <screenshot> tag"
     def __init__(self) -> None:
         self.images: list[Image] = []
         "the list with thumbnail images"
@@ -293,15 +316,17 @@ class Screenshot:
         "The caption"
 
     def get_source_image(self) -> Optional[Image]:
+        "Returns the soucre image"
         for image in self.images:
             if image.type in ("source", None):
                 return image
         return None
 
     def get_thumbnail_images(self) -> list[Image]:
+        "Returns the thumbnail images"
         return [image for image in self.images if image.type == "thumbnail"]
 
-    def load_tag(self, tag: etree.Element):
+    def load_tag(self, tag: etree.Element) -> None:
         "Load a screenshot tag"
         for i in tag.findall("image"):
             img = Image()
@@ -312,6 +337,7 @@ class Screenshot:
 
 
 class AppstreamComponent:
+    "Represents AppStream Component"
     def __init__(self) -> None:
         self.id: str = ""
         "The component ID"
@@ -452,6 +478,7 @@ class AppstreamComponent:
             self.internet[relation] = internet_dict
 
     def parse_component_tag(self, tag: etree._ElementTree) -> None:
+        "Parses a XML tag"
         self.id = tag.find("id").text.strip()
 
         try:
@@ -581,14 +608,15 @@ class AppstreamComponent:
         custom_tag = tag.find("custom")
         if custom_tag is not None:
             for i in custom_tag.findall("value"):
-                self.custom[i.get("Key").strip()] = i.text.strip()
+                if key := i.get("key", "").strip():
+                    self.custom[key] = i.text.strip()
 
         for i in tag.findall("extends"):
             self.extends.append(i.text.strip())
 
-    def load_file(self, path: str) -> None:
+    def load_file(self, path: Union[str, os.PathLike]) -> None:
         """Load an appdata.xml or metainfo.xml file"""
-        root = etree.parse(path)
+        root = etree.parse(str(path))
         self.parse_component_tag(root)
 
     def load_bytes(self, data: bytes, encoding: Optional[str] = None) -> None:
@@ -600,10 +628,11 @@ class AppstreamComponent:
         """Load an appdata.xml or metainfo.xml string"""
         self.load_bytes(text.encode("utf-8"), encoding="utf-8")
 
-    def _get_relation_tag(self, parent_tag: etree.Element, relation: str) -> None:
+    def _get_relation_tag(self, parent_tag: etree.Element, relation: Literal["supports", "recommends", "requires"]) -> None:
+        "Craetes a relation tag from the Component."
         relation_tag = etree.SubElement(parent_tag, relation)
 
-        for key, value in self.controls:
+        for key, value in self.controls.items():
             if key in CONTROL_TYPES and value == relation:
                 control_tag = etree.SubElement(relation_tag, "control")
                 control_tag.text = key
@@ -620,6 +649,7 @@ class AppstreamComponent:
             parent_tag.remove(relation_tag)
 
     def get_component_tag(self) -> etree.Element:
+        "Creates a XML tag from the Component"
         tag = etree.Element("component")
         tag.set("type", self.type)
 
@@ -689,7 +719,7 @@ class AppstreamComponent:
 
         self._get_relation_tag(tag, "supports")
         self._get_relation_tag(tag, "recommends")
-        self._get_relation_tag(tag, "supports")
+        self._get_relation_tag(tag, "requires")
 
         if len(self.kudos) > 0:
             kudos_tag = etree.SubElement(tag, "kudos")
